(*
 *  beware ircd, Internet Relay Chat server, bwinnt.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

{
code for NT system service
}

unit bwinnt;

interface

uses
  {$ifdef fpc}bmessages,{$else}messages,winsvc,{$endif}
  windows,sysutils;

const
  outputfile='output.txt';
  servicename:string='bewareserv_old';

Const { from WINNT.H }
  Service_Win32_Own_Process   = $10;
  Service_Win32_Share_Process = $20;
  {servicename='BewareIRCD';}

  SERVICE_AUTO_START=  $00000002;
  SERVICE_DEMAND_START=$00000003;
  SERVICE_ERROR_NORMAL=$00000001;
  EVENTLOG_ERROR_TYPE=$0001;

Var
  StatusHandle : Service_Status_Handle;

  runasservice:boolean = false;

procedure InstallService(auto:boolean);
procedure unInstallService;
function runservice:boolean;

procedure reportstopped;
procedure reportstopping;

procedure markntservice(installed:boolean);
function ntservicemarked:boolean;

{procedure logwrite(const s:string);}

implementation

uses btime,bircservunit,wcore;

const
  markerfile='bircserv.nt';

(*procedure logwrite(const s:string);
var
  t:textfile;
begin
  assignfile(t,'log.txt');
  {$i-}append(t);{$i+}if ioresult <> 0 then rewrite(t);
  writeln(t,s);
  closefile(t);
end;*)


procedure markntservice(installed:boolean);
var
  t:textfile;
begin
  if installed then begin
    assign(t,markerfile);
    {$i-}rewrite(t);{$i+}
    if ioresult <> 0 then exit;
    writeln(t,'The presence of this file causes this program to attempt to run as NT service.');
    closefile(t);
  end else begin
    assign(t,markerfile);
    {$i-}erase(t);{$i+}
  end;
end;

function ntservicemarked:boolean;
var
  t:textfile;
begin
  assign(t,markerfile);
  {$i-}reset(t);{$i+}
  result := ioresult = 0;
end;

procedure bSetServiceStatus(StatusHandle:Service_Status_Handle;var Status:Tservicestatus);
begin
  SetServiceStatus(StatusHandle,{$ifdef fpc}@{$endif}Status);
end;

procedure reportstopping;
Var
  Status : TServiceStatus;
begin
  if not runasservice then exit;
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Stop_pending;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  bSetServiceStatus(StatusHandle,Status);
end;

procedure reportstopped;
Var
  Status : TServiceStatus;
begin
  if not runasservice then exit;
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Stopped;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  bSetServiceStatus(StatusHandle,Status);
end;

Procedure MyCtrlHandler(OpCode : Integer); StdCall;
Var
  Status : TServiceStatus;
Begin
  Case OpCode of
    Service_Control_Stop        : Begin
                                    form1.diereason := 'NT service stop';
                                    form1.formclose;
                                  End;
    Service_Control_Interrogate : Begin { report our current status }
                                    With Status do Begin
                                      FillChar(Status,SizeOf(Status),0);
                                      dwServiceType := Service_Win32_Own_Process;
                                      dwCurrentState := Service_Running;
                                      dwControlsAccepted := Service_Accept_Stop;
                                    End;
                                    bSetServiceStatus(StatusHandle,Status);
                                  End;

    End;
End;


procedure servicemain(ArgCount : Integer; Args : Pointer); StdCall;
var
  Status : TServiceStatus;
begin
  StatusHandle := RegisterServiceCtrlHandler(@servicename[1],@MyCtrlHandler);
  if statushandle = 0 then exit;
  runasservice := true;
  { Report status to Service Control Manager (SCM) }
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Running;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  bSetServiceStatus(StatusHandle,Status);
  runapplication;
end;


procedure InstallService(auto:boolean);
var
   schService:SC_HANDLE;
   schSCManager:SC_HANDLE;
   lpszPath:array[0..511] of char;
   t:textfile;
   howtostart:integer;
begin
     if auto then howtostart := SERVICE_AUTO_START
     else howtostart := SERVICE_DEMAND_START;

     assignfile(t,outputfile);
     rewrite(t);

     if GetModuleFileName(0,@lpszPath,sizeof(lpszPath))=0 then exit;

     schSCManager:=OpenSCManager(nil,nil,SC_MANAGER_ALL_ACCESS);
     if (schSCManager>0) then
     begin
          schService:=CreateService(schSCManager,@ServiceName[1],@ServiceName[1],
          SERVICE_ALL_ACCESS,SERVICE_WIN32_OWN_PROCESS,howtostart,
          SERVICE_ERROR_NORMAL,@lpszPath,nil,nil,nil,nil,nil);
          if (schService>0) then
          begin
               markntservice(true);
               Writeln(t,'Install Ok.');
               CloseServiceHandle(schService);
          end
          else
               Writeln(t,'Unable to install '+ServiceName+', CreateService Fail. (service is already installed)');
     end
     else
         Writeln(t,'Unable to install '+ServiceName+', OpenSCManager Fail.');
    closefile(t);
end;

procedure UnInstallService;
var
  ServiceControlHandle: SC_HANDLE;
  SCManagerHandle: SC_HANDLE;
  s:string;
  Status : TServiceStatus;
  t:textfile;
begin
  assignfile(t,outputfile);
  rewrite(t);

  s := servicename;

  SCManagerHandle := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if (SCManagerHandle > 0) then begin
    ServiceControlHandle := OpenService(SCManagerHandle, @s[1], SERVICE_ALL_ACCESS);
    if (ServiceControlHandle > 0) then begin
      if ControlService(ServiceControlHandle, SERVICE_CONTROL_STOP, {$ifdef fpc}@{$endif}Status) then begin
        Write(t,'Stopping Service ');
        Sleep(1000);
        while (QueryServiceStatus(ServiceControlHandle, {$ifdef fpc}@{$endif}Status)) do begin
          if Status.dwCurrentState = SERVICE_STOP_PENDING then begin
            Write(t,'.');
            Sleep(1000);
          end else
            break;
        end;
        WriteLn(t);

        if Status.dwCurrentState = SERVICE_STOPPED then
          writeLn(t,'Service Stop Now')
        else begin
          CloseServiceHandle(ServiceControlHandle);
          CloseServiceHandle(SCManagerHandle);

          WriteLn(t,'Service Stop Fail');
          closefile(t);
          exit;
        end;
      end;
      if (DeleteService(ServiceControlHandle)) then begin
        markntservice(false);
        WriteLn(t,'Service Uninstall Ok.')
      end
      else
        WriteLn(t,'DeleteService fail');
      CloseServiceHandle(ServiceControlHandle);
    end else
      WriteLn(t,'OpenService fail (service is not installed)');
    CloseServiceHandle(SCManagerHandle);
  end else
    WriteLn(t,'OpenSCManager fail');
    closefile(t);
end;

Var
  STEs : Array[0..1] of TServiceTableEntry;

function runservice:boolean;
begin
  With STEs[0] do Begin
    lpServiceName := @servicename[1];
    lpServiceProc := @servicemain;
  End;
  With STEs[1] do Begin
    lpServiceName := nil;
    lpServiceProc := nil;
  End;
  runasservice := false;
  StartServiceCtrlDispatcher({$ifdef fpc}@{$endif}STEs[0]);
  result := runasservice;
end;


end.
